@@ -37,6 +37,7 @@ typedef enum {
   IMAP_CONNECT_FAILED, /* transport level connect failed */
   IMAP_PROTOCOL_ERROR, /* usually unexpected server response */
   IMAP_AUTH_FAILURE, /* authentication failure */
+  IMAP_AUTH_CANCELLED, /* authentication cancelled, do not retry */
   IMAP_AUTH_UNAVAIL, /* no supported authentication method available */
   IMAP_UNSECURE,     /* secure connection was requested but could not
                       * be established. */
@@ -134,6 +135,7 @@ typedef struct ImapEnvelope_ {
   gchar *message_id;
 } ImapEnvelope;
 
+/* header fetching types */
 typedef enum {
   IMFETCH_BODYSTRUCT = 1<<0,
   IMFETCH_ENV        = 1<<1,
@@ -149,6 +151,13 @@ typedef enum {
                           IMFETCH_LIST_POST)
 } ImapFetchType;
 
+/* body fetching option */
+typedef enum {
+  IMFB_NONE,
+  IMFB_HEADER, /* preceed with BODY[section.HEADER] */
+  IMFB_MIME    /* preceed with BODY[section.MIME]   */
+} ImapFetchBodyOptions;
+
 typedef struct ImapBody_ ImapBody;
 
 typedef enum { IMBMEDIA_MULTIPART,
@@ -231,9 +240,13 @@ typedef struct _ImapMboxHandle      ImapMboxHandle;
 
 ImapEnvelope *imap_envelope_new(void);
 void imap_envelope_free(ImapEnvelope *);
+gchar* imap_envelope_to_string(const ImapEnvelope* env);
+ImapEnvelope* imap_envelope_from_string(const gchar *s);
 
 ImapBody* imap_body_new(void);
 void imap_body_free(ImapBody* body);
+gchar* imap_body_to_string(const ImapBody *body);
+ImapBody* imap_body_from_string(const gchar *s);
 
 void imap_body_set_desc(ImapBody* body, char* str);
 void imap_body_add_param(ImapBody *body, char *key, char *val);
@@ -248,7 +261,8 @@ typedef enum {
   IME_GET_USER,
   IME_TLS_VERIFY_ERROR,
   IME_TLS_NO_PEER_CERT,
-  IME_TLS_WEAK_CIPHER
+  IME_TLS_WEAK_CIPHER,
+  IME_TIMEOUT
 } ImapUserEventType;
 
 typedef void (*ImapUserCb)(ImapUserEventType ue, void *arg, ...);
@@ -259,6 +273,7 @@ void imap_mbox_handle_msg_deserialize(ImapMboxHandle *h, unsigned msgno,
                                       void *data);
 void*        imap_message_serialize(ImapMessage *);
 ImapMessage* imap_message_deserialize(void *data);
+size_t imap_serialized_message_size(void *data);
 
 const char *lbi_strerror(ImapResult rc);
 
