@@ -136,7 +136,6 @@ typedef enum {
     LB_MAILBOX_STATE_CLOSED,
     LB_MAILBOX_STATE_OPENING,
     LB_MAILBOX_STATE_OPEN,
-    LB_MAILBOX_STATE_TREECLEANING,
     LB_MAILBOX_STATE_CLOSING
 } LibBalsaMailboxState;
 
@@ -225,6 +224,12 @@ struct _LibBalsaMailbox {
     LibBalsaCondition *view_filter; /* to choose a subset of messages
                                      * to be displayed, e.g., only
                                      * undeleted. */
+    LibBalsaCondition *persistent_view_filter; /* the part of the view 
+                                                * filter that will persist 
+                                                * to the next time the
+                                                * mailbox is opened */
+    gboolean view_filter_pending;  /* a view filter has been set
+                                    * but the view has not been updated */
 
     /* info fields */
     gboolean has_unread_messages;
@@ -244,6 +249,11 @@ struct _LibBalsaMailbox {
 
     /* Whether the tree has been changed since some event. */
     gboolean msg_tree_changed;
+
+#ifdef BALSA_USE_THREADS
+    /* Array of msgnos that need to be displayed. */
+    GArray *msgnos_pending;
+#endif                          /* BALSA_USE_THREADS */
 };
 
 /* Search iter */
@@ -254,6 +264,13 @@ struct _LibBalsaMailboxSearchIter {
     gpointer user_data;		/* private backend info */
 };
 
+/** Iterates over a list of messages, returning each time it is called
+    flags and the stream to a message. It is the responsibility of the
+    called to un-ref the stream after use. */
+typedef gboolean (*LibBalsaAddMessageIterator)(LibBalsaMessageFlag *,
+					       GMimeStream **stream,
+					       void *);
+
 struct _LibBalsaMailboxClass {
     GObjectClass parent_class;
 
@@ -267,8 +284,7 @@ struct _LibBalsaMailboxClass {
     gboolean (*open_mailbox) (LibBalsaMailbox * mailbox, GError **err);
     void (*close_mailbox) (LibBalsaMailbox * mailbox, gboolean expunge);
     LibBalsaMessage *(*get_message) (LibBalsaMailbox * mailbox, guint msgno);
-    void (*prepare_threading)(LibBalsaMailbox *mailbox, guint * msgnos,
-                              guint len);
+    gboolean (*prepare_threading)(LibBalsaMailbox *mailbox, guint start);
     gboolean (*fetch_message_structure)(LibBalsaMailbox *mailbox,
                                         LibBalsaMessage * message,
                                         LibBalsaFetchFlag flags);
@@ -280,7 +296,7 @@ struct _LibBalsaMailboxClass {
 				  LibBalsaMessageBody *part,
                                   GError **err);
     GMimeStream *(*get_message_stream) (LibBalsaMailbox * mailbox,
-                                        guint msgno);
+                                        guint msgno, gboolean peek);
 
     void (*check) (LibBalsaMailbox * mailbox);
 
@@ -293,9 +309,9 @@ struct _LibBalsaMailboxClass {
     void (*save_config) (LibBalsaMailbox * mailbox, const gchar * prefix);
     void (*load_config) (LibBalsaMailbox * mailbox, const gchar * prefix);
     gboolean (*sync) (LibBalsaMailbox * mailbox, gboolean expunge);
-    gboolean (*add_message) (LibBalsaMailbox * mailbox,
-                             GMimeStream * stream,
-                             LibBalsaMessageFlag flags, GError ** err);
+    guint (*add_messages) (LibBalsaMailbox * mailbox,
+			   LibBalsaAddMessageIterator msg_iterator,
+			   void *iter_arg, GError ** err);
     gboolean (*messages_change_flags) (LibBalsaMailbox * mailbox,
 				       GArray *msgnos,
 				       LibBalsaMessageFlag set,
@@ -358,9 +374,11 @@ LibBalsaMessage *libbalsa_mailbox_get_message(LibBalsaMailbox * mailbox,
 /** libbalsa_mailbox_prepare_threading() requests prefetching of information
     needed for client-side message threading.
     msgnos are related to currently set view.
+    Returns TRUE if successful; FALSE may mean that the mailbox was
+    closed during the operation.
 */
-void libbalsa_mailbox_prepare_threading(LibBalsaMailbox *mailbox,
-					guint * msgnos, guint len);
+gboolean libbalsa_mailbox_prepare_threading(LibBalsaMailbox * mailbox,
+                                            guint start);
 
 /** libbalsa_mailbox_fetch_message_structure() fetches detailed
     message structure for given message. It can also fetch all RFC822
@@ -394,7 +412,7 @@ gboolean libbalsa_mailbox_get_message_part(LibBalsaMessage    *message,
     with full RFC822 text of the message.
 */
 GMimeStream *libbalsa_mailbox_get_message_stream(LibBalsaMailbox * mailbox,
-						 guint msgno);
+						 guint msgno, gboolean peek);
 
 
 /** libbalsa_mailbox_sync_storage() asks the mailbox to synchronise
@@ -443,6 +461,12 @@ gboolean libbalsa_mailbox_add_message(LibBalsaMailbox * mailbox,
                                       GMimeStream * stream,
                                       LibBalsaMessageFlag flags,
                                       GError ** err);
+
+guint libbalsa_mailbox_add_messages(LibBalsaMailbox * mailbox,
+				    LibBalsaAddMessageIterator msg_iterator,
+				    void *arg,
+				    GError ** err);
+
 gboolean libbalsa_mailbox_close_backend(LibBalsaMailbox * mailbox);
 
 /* Message number-list methods */
@@ -457,7 +481,6 @@ gboolean libbalsa_mailbox_messages_move(LibBalsaMailbox * mailbox,
 					GArray * msgnos,
 					LibBalsaMailbox * dest, GError **err);
 
-                               
 /*
  * misc mailbox releated functions
  */
@@ -477,12 +500,16 @@ LibBalsaMailboxIndexEntry* libbalsa_mailbox_index_entry_new_from_msg
                            (LibBalsaMessage *msg);
 void libbalsa_mailbox_index_entry_set_no(LibBalsaMailboxIndexEntry *entry,
                                          unsigned no);
-void libbalsa_mailbox_index_entry_free(LibBalsaMailboxIndexEntry *entry);
+void libbalsa_mailbox_index_entry_clear(LibBalsaMailbox * mailbox,
+                                        guint msgno);
 void libbalsa_mailbox_index_set_flags(LibBalsaMailbox *mailbox,
 				      unsigned msgno, LibBalsaMessageFlag f);
-void libbalsa_mailbox_set_view_filter(LibBalsaMailbox   *mailbox,
-                                      LibBalsaCondition *filter_condition,
-                                      gboolean update_immediately);
+gboolean libbalsa_mailbox_set_view_filter(LibBalsaMailbox * mailbox,
+                                          LibBalsaCondition *
+                                          filter_condition,
+                                          gboolean update_immediately);
+void libbalsa_mailbox_make_view_filter_persistent(LibBalsaMailbox *
+                                                  mailbox);
 
 gboolean libbalsa_mailbox_can_do(LibBalsaMailbox *mailbox,
                                  enum LibBalsaMailboxCapability cap);
@@ -575,6 +602,11 @@ gboolean libbalsa_mailbox_msgno_find(LibBalsaMailbox * mailbox,
 				     guint seqno,
 				     GtkTreePath ** path,
 				     GtkTreeIter * iter);
+/* Manage message flags */
+gboolean libbalsa_mailbox_msgno_change_flags(LibBalsaMailbox * mailbox,
+                                             guint msgno,
+                                             LibBalsaMessageFlag set,
+                                             LibBalsaMessageFlag clear);
 /* Test message flags */
 gboolean libbalsa_mailbox_msgno_has_flags(LibBalsaMailbox * mailbox,
                                           guint seqno,
@@ -599,7 +631,7 @@ void libbalsa_mailbox_set_encr_icon(GdkPixbuf * pixbuf);
 void libbalsa_mailbox_try_reassemble(LibBalsaMailbox * mailbox,
 				     const gchar * id);
 
-/* Message number arrays */
+/* Message numbers and arrays */
 void libbalsa_mailbox_register_msgnos(LibBalsaMailbox * mailbox,
 				      GArray * msgnos);
 void libbalsa_mailbox_unregister_msgnos(LibBalsaMailbox * mailbox,
@@ -642,7 +674,6 @@ typedef enum {
     LB_MBOX_SIZE_COL,
     LB_MBOX_WEIGHT_COL,
     LB_MBOX_STYLE_COL,
-    LB_MBOX_MESSAGE_COL,
     LB_MBOX_N_COLS
 } LibBalsaMailboxColumn;
 
