@@ -27,7 +27,7 @@
 #include "misc.h"
 #include "balsa-message.h"
 #include "quote-color.h"
-#include "i18n.h"
+#include <glib/gi18n.h>
 #include "balsa-print-object.h"
 #include "balsa-print-object-decor.h"
 #include "balsa-print-object-header.h"
